/*
 	led_radio.ino	LED周波数表示付きFMラジオ
 			Programmed by ハンダマスターかしま 2022/5/8
 			The MIT License (MIT)
 			Copyright (c) 2022 H.Kashima

	Description: 
		Arduno Nano Everyで動作する NeoPixcelドットマトリクス周波数表示付き RDA5807 FMラジオ
	Board Library:
		Arduino megaAVR Borads
	Library:
		Adafruit_NeoMatrix
		Radio
	更新履歴:
 		2022.5.8 初版

   ----- 使用ピン -----
   D2	in pullup	UPスイッチ
   D3	in pullup	DOWNスイッチ
   A4	out			SDA (RDA5807)
   A5	out			SCL (RDA5807)
   D10	out			NeoPixelデーター
*/
#include <Adafruit_NeoMatrix.h>
#include "RDA5807Mex.h"				// private宣言解除版 RDA5807M.h
#include "bitmap.h"					// ロゴ画像データ

#define DEBUG
//
// プログラム設定値
//
#define UP_PIN		2				// UPボタン
#define DOWN_PIN	3				// DOWNボタン
#define DATA_PIN	10				// NeoPixel LED信号出力ピン
#define LED_COLS	32				// マトリクスサイズ横
#define LED_ROWS	8				// マトリクスサイズ縦

#define	TEXT_COLOR	matrix.Color( 192,64, 0)		// 周波数の表示色
#define ANT_COLOR	matrix.Color( 0, 32, 96)		// アンテナの色

//
// radioオブジェクト
//
class RDA5807Mex : public RDA5807M {				// RDA5807Mを元に派生オブジェクトを生成
	public:
		// 局サーチRSSIしきい値の設定
		void setSeekTh(uint16_t v){
			registers[5] &= 0xf0ff;					// 局選択しきい値をクリア
			registers[5] |= (v & 0x0f) << 8;		// しきい値を設定
			_saveRegister(5);						// レジスタ保存
		}
} radio;

//
// matrixオブジェクト
//
Adafruit_NeoMatrix matrix = Adafruit_NeoMatrix(
				LED_COLS, LED_ROWS, DATA_PIN,				// 縦, 横, PWMデーターピン
				NEO_MATRIX_TOP     + NEO_MATRIX_LEFT +		// 左上が最初のLED
				NEO_MATRIX_COLUMNS + NEO_MATRIX_ZIGZAG,		// LEDはカラム方向へジグザグ配置
				NEO_GRB            + NEO_KHZ800 );			// ws2812bフォーマット, 400KHz転送

//
// 初期設定処理
//
void setup() {
	pinMode( UP_PIN, INPUT_PULLUP);			// ピン設定
	pinMode( DOWN_PIN, INPUT_PULLUP);
	Serial.begin(9600);
	delay(200);
	Serial.println();
	String str = "RL FM Radio v1";
	Serial.println(str);

	//radio.debugEnable();
 	radio.init();							// RDA5807M初期化

	//
	// スタートアップを表示
	//
	matrix.begin();							// NeoMatrixを開始
	matrix.setBrightness(32);				// 輝度の設定
	matrix.setTextWrap(false);
	matrix.setTextColor(TEXT_COLOR);
	// 画面の右隅から文字列の最後が右隅にくるまでスクロール
	for(int x = matrix.width(); x > (LED_COLS - (int)str.length() * 6); x--){
		matrix.fillScreen(0);
		matrix.setCursor(x, 0);
		matrix.print(str);
		matrix.show();
		delay(10);
	}
	matrix.show();
	delay(1000);

	radio.setSeekTh(14);					// サーチしきい値 14
 	radio.setVolume(15);					// 音声出力 15 (0～15)
 	radio.setMute(false);					// ミュート解除
 	radio.setMono(false);					// Mono強制解除
	delay(1000);
	matrix.fillScreen(0);					// 画面消去
	matrix.show();							// LEDへ出力

 	radio.setBandFrequency( RADIO_BAND_FMWORLD, 7950);
}

//
// ボタン状態推移変数
//
bool g_btnUP = false, g_btnDOWN = false;
bool g_btnUP_long = false, g_btnDOWN_long = false;
unsigned long g_btnUP_prs = 0, g_btnUP_rel = 0;
unsigned long g_btnDOWN_prs = 0, g_btnDOWN_rel = 0;

bool g_update = false;		// 強制再表示フラグ
bool g_followUpd = false;	// 周波数設定遅れのための再表示フォローフラグ
bool g_seeking = false;		// シーク中フラグ
bool g_BlinkMode = false;	// アンテナ表示OFFフラグ

//
// メインループ
//
void loop() {
	bool b_AllPushReq = false;
	char mhz[4], khz[2];		// 周波数表示文字
	int pass = 0;				// 自動表示カウンタ

	while(1){
		//
		// UPボタン処理
		//
		if(g_btnUP == false && g_btnUP_long == false && digitalRead(UP_PIN) == LOW){
			// 何もしていない状態でボタンが押されたら
			g_btnUP_rel = 0; g_btnUP_prs = millis();			// 解除時刻クリア, 押下時刻保存
			uint16_t f = radio.getFrequency();					// 現在周波数の読出し
			f += 10;											// +100k
			if( f > 10800 )										// 周波数限界を超えたか
				f = 7600;
			radio.setFrequency(f);								// 受信周波数を設定
			if(g_BlinkMode)										// 常時表示OFFの場合、実周波数が遅れて
				g_followUpd = true;								// 表示とずれる場合があるので1発だけ表示
			g_btnUP = true; g_update = true;					// 押下フラグON, 再表示要求
			Serial.println("UP Pushed");

		} else if(g_btnUP_long == false && g_btnUP == true && digitalRead(UP_PIN) == LOW){
			if( digitalRead(DOWN_PIN) == LOW ){					// DOWN同時押し検出
				b_AllPushReq = true;

			} else {
				// UPが押された状態でさらに押下状態だったら
				long duration = millis() - g_btnUP_prs;			// 最初に押された時刻からの時間を計算
				if( duration > 2000){							// それが 2秒以上の時
					radio.seekUp();								// 受信局サーチ開始
					g_seeking = true;							// シークフラグON
					g_btnUP = false; g_btnUP_long = true;		// 長押しフラグON
					Serial.println("UP long pressed");
				}
			}

		} else if( digitalRead(UP_PIN) == HIGH && (g_btnUP == true || g_btnUP_long == true)){
			// 長押し, 単純押しフラグONの時ボタンが放されたら
			g_btnUP_prs = 0; g_btnUP_rel = millis();			// 押下時刻クリア, 解除時刻保存
			g_btnUP = false; g_btnUP_long = false;				// 押下げフラグクリア
			Serial.println("UP Released");

		//
		// DOWNボタン処理
		//
		} else if(g_btnDOWN == false && g_btnDOWN_long == false && digitalRead(DOWN_PIN) == LOW){
			g_btnDOWN_rel = 0; g_btnDOWN_prs = millis();
			uint16_t f = radio.getFrequency();
			f -= 10;							// -100k
			if( f < 7600 )
				f = 10800;
			radio.setFrequency(f);
			if(g_BlinkMode)
				g_followUpd = true;
			g_btnDOWN = true; g_update = true;
			Serial.println("DOWN Pushed");

		} else if(g_btnDOWN_long == false && g_btnDOWN == true && digitalRead(DOWN_PIN) == LOW){
			if( digitalRead(UP_PIN) == LOW ){	// UP同時押し検出
				b_AllPushReq = true;

			} else {
				long duration = millis() - g_btnDOWN_prs;
				if( duration > 2000){
					radio.seekDown();
					g_seeking = true;
					g_btnDOWN = false; g_btnDOWN_long = true;
					Serial.println("DOWN long pressed");
				}
			}

		} else if( digitalRead(DOWN_PIN) == HIGH && (g_btnDOWN == true || g_btnDOWN_long == true)){
			g_btnDOWN_prs = 0; g_btnDOWN_rel = millis();
			g_btnDOWN = false; g_btnDOWN_long = false;
			Serial.println("DOWN Released");

		}

		// 同時押しの処理
		if( b_AllPushReq && g_btnUP == false && g_btnDOWN == false ){
			b_AllPushReq = false;
			if( g_BlinkMode ){				// アンテナ表示ON/OFF
				g_BlinkMode = false;
				Serial.println("BL OFF");
			} else {
				g_BlinkMode = true;
				Serial.println("BL ON");
			}
			g_update = true;
		}
		//
		// 周波数表示
		//
		if( ++pass > 50 || g_update || g_followUpd){	// メインループ 50回に 1回表示処理 (0.5秒毎)
														// 但し再表示要求の時はすぐに表示
			pass = 0;
			uint16_t f = radio.getFrequency();			// 現在受信周波数(レジスタ0x0A)の取得
			RADIO_INFO info;
			radio.getRadioInfo(&info);					// 受信ステータスの取得
			sprintf( mhz, "%5d", f);
			khz[0] = mhz[3]; khz[1] = 0;				// 周波数値(100倍, 80.0MHzなら8000)を
			mhz[3] = 0;									// 小数点を付けて文字配列へ変換

			matrix.fillScreen(0);						// 画面消去

			//
			// ロゴ登録の検索
			int i;
			for(i = 0; i < sizeof(logos)/sizeof(*logos); i++){		// 要素数最大まで繰返し
				if(logos[i].freq == f){					// 周波数値発見
					Serial.print("LOGO ");
					break;
				}
				if(logos[i].freq == 0) break;			// テーブル最後に到達
			}

			if(logos[i].freq == 0){						// ロゴ登録なし
				matrix.setCursor(0, 0);
				matrix.print(mhz);						// 周波数を文字表示
				matrix.drawPixel( 18, 6, TEXT_COLOR);
				matrix.setCursor(20, 0);
				matrix.print(khz);

			} else {									// ロゴを表示
				matrix.drawRGBBitmap( 0, 0, logos[i].bitmap, 26, 8);

			}

			if(g_BlinkMode == false){						// アンテナ表示OFF時は表示しない
				// ステレオアイコンの処理
				matrix.fillRect(27, 0, 6, 8, 0);			// 表示領域を消去
				if(info.stereo)								// STEREOフラグでロゴ表示
					matrix.drawRGBBitmap( 27, 0, icon_stereo, 5, 4);
	
				// 受信レベル(RSSI)の処理
				if(info.rssi > 19)							// 受信レベル 20でアンテナ 1本
					matrix.drawLine(27, 6, 27, 7, ANT_COLOR);
				if(info.rssi > 24)							// 25で 2本
					matrix.drawLine(29, 5, 29, 7, ANT_COLOR);
				if(info.rssi > 29)							// 30で 3本
					matrix.drawLine(31, 4, 31, 7, ANT_COLOR);
			}
				
			
			// シリアルへデバッグ出力
			if(g_BlinkMode) Serial.print("BL ");
			Serial.print(mhz); Serial.print("."); Serial.print(khz);
			String str = " : ";
			if(info.stereo) str += "STEREO ";
			str += String(info.rssi) + " ";
			if(info.tuned) str += "TUNED ";
			if(info.mono) str += "MONO ";
			str += radio.registers[10] & 0x4000 ? "STC " : "";
			str += radio.registers[10] & 0x2000 ? "SF " : "";
			Serial.println(str);

			if(g_followUpd || g_update || g_BlinkMode == false ||
				(radio.registers[10] & 0x4000) == 0){			// アンテナ表示OFF時はアップデート要求とSEEK中以外は表示しない

				matrix.show();									// LEDへデーター転送
				if(g_followUpd && g_update == false)
					g_followUpd = false;
			}

			if(g_seeking && (radio.registers[10] & 0x4000)){	// シーク終了の検出
				g_seeking = false;
				g_followUpd = true;
				Serial.println("Seek complete");
			}			

			g_update = false;
		}

		delay(10);										// ループ時間待ち 10ms
	}
}
